A5. RChUn̗͂ƃgNєSx̌vZFORTRANTu[`

00010 C********************************************************************* 
00020 C*    THIS SUBROUTINE IS FOR CALCULATING FORCES AND TORQUES          * 
00030 C*    FOR A PARTICLE MODEL WITH A POINT DIPOLE MOMENT.               * 
00040 C*-------------------------------------------------------------------* 
00050 C*    N     : NUMBER OF PARTICLES                                    * 
00060 C*    D     : DIAMETER OF PARTICLE INCLUDING SURFACTANT LAYER        * 
00070 C*            ( =2 FOR THIS CASE )                                   * 
00080 C*    NDENS : NUMBER DENSITY                                         * 
00090 C*    RM    : NONDIMENSIONAL PARAMETER OF PARTICLE-PARTICLE INTERACT * 
00100 C*            (MAGNETIC PARTICLE-PARTICLE FORCE) / (VISCOUS FORCE)   * 
00110 C*    RH    : NONDIMENSIONAL PARAMETER OF FIELD-PARTICLE INTERACT    * 
00120 C*            (MAGNETIC FIELD-PARTICLE FORCE) / (VISCOUS FORCE)      * 
00130 C*    RV    : NONDIMENSIONAL PARAMETER OF PARTICLE-PARTICLE INTERACT * 
00140 C*            (STERIC PARTICLE-PARTICLE FORCE) / (VISCOUS FORCE)     * 
00150 C*    RCOFF : CUTOFF RADIUS FOR CALCULATION OF INTERACTION           * 
00160 C*    RCOFF2: 2*RCOFF                                                * 
00170 C*    RCOFFMOB : CUTOFF RADIUS FOR CALCULATION OF MOBILITY FUNCTIONS * 
00180 C*    L     : DIMENSIONS OF SIMULATION REGION                        * 
00190 C*    H     : TIME INTERVAL FOR SIMULATIONS                          * 
00200 C*    TM    : THICKNESS OF SURFACTANT LAYER                          * 
00210 C*    (HX,HY,HZ)        : APPLIED MAGNETIC FIELD (UNIT VECTOR)       * 
00220 C*                                                                   * 
00230 C*    RX(N),RY(N),RZ(N) : PARTICLE POSITION (-L/2<RX,RY,RZ<L/2)      * 
00240 C*    NX(N),NY(N),NZ(N) : DIRECTION OF MAGNETIC MOMENT               * 
00250 C*    FX(N),FY(N),FZ(N) : PARTICLE FORCE                             * 
00260 C*    TORQX(N),TORQY(N),TORQZ(N) : PARTICLE TORQUE                   * 
00270 C*    VISYXMM,VISYXMH,VISXYMM,VISXYMH : VISCOSITIES                  * 
00280 C*    DX,CORY : LEES-EDWARDS BOUNDARY CONDITION                      * 
00290 C*********************************************************************
00300 C  
00310 C            --- CALCULATE FORCES AND TORQUES, WITHOUT VISCOSITIES ---
00320 CCC   ITREE = 1
00330 CCC   CALL FORCE( RCOFF2,ITREE,VISYXMM,VISYXMH,VISXYMM,VISXYMH )
00340 C                    --- CALCULATE FORCES, TORQUES AND VISCOSITIES ---
00350 CCC   ITREE = 2
00360 CCC   CALL FORCE( RCOFF2,ITREE,VISYXMM,VISYXMH,VISXYMM,VISXYMH )
00370 C  
00380 C**** SUB FORCE *****                                                  
00390       SUBROUTINE FORCE( RCOFF2,ITREE,VISYXMM,VISYXMH,VISXYMM,VISXYMH )
00400 C                                                                      
00410       IMPLICIT REAL*8 (A-H,O-Z)                                        
00420 C                                                                      
00430       COMMON /BLOCK1/  RX , RY , RZ                                    
00440       COMMON /BLOCK2/  NX , NY , NZ                                    
00450       COMMON /BLOCK5/  FX , FY , FZ 
00460       COMMON /BLOCK6/  TORQX , TORQY , TORQZ 
00470       COMMON /BLOCK7/  N , NDENS , RCOFF ,  L , H , TM , D , RCOFFMOB 
00480       COMMON /BLOCK8/  RM , RH , RV , HX , HY , HZ
00490       COMMON /BLOCK19/ DX , CORY                  
00500 C
00510       INTEGER  NN                                                   
00520       PARAMETER( NN=1000 , PI=3.141592653589793D0 )
00530 C                                                                      
00540 C                                                                      
00550       REAL*8   RX(NN) , RY(NN) , RZ(NN) , NX(NN) , NY(NN) , NZ(NN)     
00560       REAL*8   FX(NN) , FY(NN) , FZ(NN)
00570       REAL*8   TORQX(NN) , TORQY(NN) , TORQZ(NN) 
00580       REAL*8   NDENS  , L      
00590 C                                                                      
00600       REAL*8   RXI , RYI , RZI , RXIJ , RYIJ , RZIJ
00610       REAL*8   NXI , NYI , NZI , NXJ  , NYJ  , NZJ 
00620       REAL*8   FXI , FYI , FZI , FXIJ , FYIJ , FZIJ
00630       REAL*8   TORQXI , TORQYI , TORQZI , TORQXIJ , TORQYIJ , TORQZIJ 
00640       REAL*8   TXIJ , TYIJ , TZIJ , RIJ , RIJ2 , RIJ3 , RIJ4 
00650       REAL*8   RM8, RMN, RMN2   
00660       REAL*8   C0 , C1 , C2 , C3 , C1X , C1Y , C1Z , C2X , C2Y, C2Z 
00670 C 
00680       RM8  = 8.D0*RM
00690       RMN  = 2.D0/(1.D0+TM)
00700       RMN2 = RMN**2
00710       DO 10 I=1,N                                  
00720         FX(I) = 0.D0                               
00730         FY(I) = 0.D0                               
00740         FZ(I) = 0.D0                               
00750         TORQX(I) = 0.D0 
00760         TORQY(I) = 0.D0 
00770         TORQZ(I) = 0.D0 
00780    10 CONTINUE             
00790       IF( ITREE .EQ. 2 ) THEN
00800         VISYXMM = 0.D0                             
00810         VISYXMH = 0.D0                             
00820         VISXYMM = 0.D0                             
00830         VISXYMH = 0.D0
00840       END IF                                              
00850 C 
00860 C                                                                      
00870       DO 100 I=1,N-1                                                   
00880 C                                                                      
00890         RXI = RX(I)                                                    
00900         RYI = RY(I)                                                    
00910         RZI = RZ(I)                                                    
00920         NXI = NX(I)                                                    
00930         NYI = NY(I)                                                    
00940         NZI = NZ(I)                                                    
00950         FXI = FX(I)                                                    
00960         FYI = FY(I)                                                    
00970         FZI = FZ(I)                                                    
00980         TORQXI = TORQX(I)   
00990         TORQYI = TORQY(I)   
01000         TORQZI = TORQZ(I)   
01010 C                           
01020         DO 50 J=I+1,N       
01030 C
01040           RZIJ = RZI  - RZ(J)                                          
01050           RZIJ = RZIJ - DNINT(RZIJ/L)*L                               
01060           IF( DABS(RZIJ) .GE. RCOFF )      GOTO 50                     
01070           RXIJ  = RXI  - RX(J)                                          
01080           RYIJ  = RYI  - RY(J)                                          
01090           CORY  = - DNINT( RYIJ/L )
01100           RYIJ  = RYIJ + CORY*L 
01110           IF( DABS(RYIJ) .GE. RCOFF )      GOTO 50                     
01120           RXIJ  = RXIJ + CORY*DX 
01130           RXIJ  = RXIJ - DNINT( RXIJ/L )*L
01140           IF( DABS(RXIJ) .GE. RCOFF )      GOTO 50                     
01150 C                                                                      
01160           RIJ2  = RXIJ*RXIJ + RYIJ*RYIJ + RZIJ*RZIJ                     
01170           IF( RIJ2 .GE. RCOFF2 )           GOTO 50                     
01180 C                                                                      
01190           IF( RIJ2 .LT. RMN2 ) THEN
01200             RIJ  = DSQRT(RIJ2)
01210             RXIJ = RMN*RXIJ/RIJ 
01220             RYIJ = RMN*RYIJ/RIJ 
01230             RZIJ = RMN*RZIJ/RIJ 
01240             RIJ2 = RMN2
01250           END IF  
01260           RIJ  = DSQRT(RIJ2)
01270           RIJ3 = RIJ*RIJ2   
01280           RIJ4 = RIJ2**2 
01290           TXIJ = RXIJ/RIJ                               
01300           TYIJ = RYIJ/RIJ                                       
01310           TZIJ = RZIJ/RIJ                                       
01320           NXJ = NX(J)                                             
01330           NYJ = NY(J)                                              
01340           NZJ = NZ(J)                                          
01350 C                         
01360           C1   = NXI*NXJ   + NYI*NYJ   + NZI*NZJ      
01370           C2   = NXI*TXIJ  + NYI*TYIJ  + NZI*TZIJ     
01380           C3   = NXJ*TXIJ  + NYJ*TYIJ  + NZJ*TZIJ     
01390 C                                               --- MAGNETIC FORCE ---
01400           FXIJ = - ( RM8/RIJ4) * (  ( - C1 + 5.D0*C2*C3 )*TXIJ 
01410      &                                        - ( C3*NXI + C2*NXJ )  ) 
01420           FYIJ = - ( RM8/RIJ4) * (  ( - C1 + 5.D0*C2*C3 )*TYIJ 
01430      &                                        - ( C3*NYI + C2*NYJ )  ) 
01440           FZIJ = - ( RM8/RIJ4) * (  ( - C1 + 5.D0*C2*C3 )*TZIJ 
01450      &                                        - ( C3*NZI + C2*NZJ )  ) 
01460 C                                             --- STERIC REPULSION ---
01470           IF( RIJ .LT. 2.D0 ) THEN 
01480              C0 = DLOG( 2.D0 / RIJ )  
01490              FXIJ = FXIJ + RV*TXIJ*C0
01500              FYIJ = FYIJ + RV*TYIJ*C0
01510              FZIJ = FZIJ + RV*TZIJ*C0
01520           END IF 
01530 C
01540           FXI  = FXI  + FXIJ                                           
01550           FYI  = FYI  + FYIJ                                           
01560           FZI  = FZI  + FZIJ                                           
01570 C                                                                      
01580           FX(J) = FX(J) - FXIJ                                         
01590           FY(J) = FY(J) - FYIJ                                         
01600           FZ(J) = FZ(J) - FZIJ                                         
01610 C                                              --- PART OF TORQUES --- 
01620           C1X   =  NYI*NZJ  - NZI*NYJ
01630           C1Y   =  NZI*NXJ  - NXI*NZJ
01640           C1Z   =  NXI*NYJ  - NYI*NXJ
01650           C2X   =  NYI*TZIJ - NZI*TYIJ
01660           C2Y   =  NZI*TXIJ - NXI*TZIJ
01670           C2Z   =  NXI*TYIJ - NYI*TXIJ
01680 C
01690           TORQXIJ = - ( C1X - 3.D0*C3*C2X )/RIJ3 
01700           TORQYIJ = - ( C1Y - 3.D0*C3*C2Y )/RIJ3 
01710           TORQZIJ = - ( C1Z - 3.D0*C3*C2Z )/RIJ3 
01720           TORQXI  = TORQXI + TORQXIJ
01730           TORQYI  = TORQYI + TORQYIJ
01740           TORQZI  = TORQZI + TORQZIJ
01750 C
01760           C2X   =  NYJ*TZIJ - NZJ*TYIJ
01770           C2Y   =  NZJ*TXIJ - NXJ*TZIJ
01780           C2Z   =  NXJ*TYIJ - NYJ*TXIJ
01790           TORQX(J) = TORQX(J) - ( -C1X - 3.D0*C2*C2X )/RIJ3
01800           TORQY(J) = TORQY(J) - ( -C1Y - 3.D0*C2*C2Y )/RIJ3
01810           TORQZ(J) = TORQZ(J) - ( -C1Z - 3.D0*C2*C2Z )/RIJ3
01820 C                                             --- CAL. VISCOSITIES --- 
01830           IF( ITREE .EQ. 2 ) THEN 
01840             VISYXMM = VISYXMM + RYIJ*FXIJ 
01850             VISXYMM = VISXYMM + RXIJ*FYIJ
01860           END IF 
01870 C 
01880    50   CONTINUE                                                       
01890 C                                                                      
01900         FX(I) = FXI                                                    
01910         FY(I) = FYI                                                    
01920         FZ(I) = FZI                                                    
01930         TORQX(I) = TORQXI                           
01940         TORQY(I) = TORQYI                           
01950         TORQZ(I) = TORQZI                           
01960 C                                                                      
01970   100 CONTINUE                                                         
01980 C                                       --- FINAL FORM OF TORQUES --- 
01990       DO 120 I=1,N     
02000         NXI = NX(I)                                                    
02010         NYI = NY(I)                                                    
02020         NZI = NZ(I)                                                    
02030         C1X   =  NYI*HZ  - NZI*HY
02040         C1Y   =  NZI*HX  - NXI*HZ
02050         C1Z   =  NXI*HY  - NYI*HX
02060 C                                                         
02070         TORQX(I) = RM*TORQX(I)*2.D0 + RH*C1X                           
02080         TORQY(I) = RM*TORQY(I)*2.D0 + RH*C1Y                           
02090         TORQZ(I) = RM*TORQZ(I)*2.D0 + RH*C1Z   
02100 C                                             --- CAL. VISCOSITIES --- 
02110         IF( ITREE .EQ. 2 ) THEN 
02120           VISYXMH = VISYXMH + TORQZ(I) 
02130         END IF 
02140   120 CONTINUE                                                         
02150       IF( ITREE .EQ. 2 ) THEN 
02160         VISYXMM = - VISYXMM*(6.D0*PI)/L**3  
02170         VISXYMM = - VISXYMM*(6.D0*PI)/L**3  
02180         VISYXMH =   VISYXMH*(4.D0*PI)/L**3  
02190         VISXYMH = - VISYXMH 
02200       END IF 
02210                                                                 RETURN 
02220                                                                 END    
